
import MetalKit

class Camera {
  var near: Float = 0.1
  var far: Float = 100
  var aspect: Float = 1
  var fov: Float = radians(fromDegrees: 60)
  
  var projectionMatrix: float4x4 {
    return float4x4(projectionFov: fov,
                    near: near,
                    far: far,
                    aspect: aspect)
  }
  
  init() {}
  
  init(near: Float, far: Float, aspect: Float, fov: Float = 1.135) {
    self.near = near
    self.far = far
    self.aspect = aspect
    self.fov = fov
  }
  
  var viewMatrix = float4x4.identity()
  var transform = Transform() {
    didSet {
      let translateMatrix = float4x4(translation: [-transform.position.x, -transform.position.y, -transform.position.z])
      let translateBackMatrix = float4x4(translation: transform.position)
      let rotateMatrix = float4x4(rotation: transform.rotation)
      let scaleMatrix = float4x4(scaling: transform.scale)
      viewMatrix =  translateMatrix * translateBackMatrix * rotateMatrix * translateMatrix * scaleMatrix
    }
  }
}
